from typing import Tuple, Optional, Iterable

from v25.messaging.message import Message
from v25.messaging.subject import Subject
from v25.storage.storage import AbstractStorage


class SecureStorage(AbstractStorage):
    def __init__(self, storage: AbstractStorage, subject: Subject):
        self.storage = storage
        self.subject = subject

    def check(self, s: Subject) -> dict:
        assert self.subject == s
        return self.storage.check(s)

    def push(self, m: Message) -> None:
        assert self.subject == m.sfrom
        return self.storage.push(m)

    def edit(self, old: Message, new: Message):
        assert self.subject == old.sfrom
        return self.storage.edit(old, new)

    def delete(self, m: Message):
        assert self.subject in m.pair
        return self.storage.delete(m)

    def pull(self, pair: Tuple[Subject, Subject], params: Optional[dict] = None) -> Iterable[Message]:
        assert self.subject in pair
        return self.storage.pull(pair, params)

    def flags(self, m: Message, flags: str):
        assert self.subject in m.pair
        return self.storage.flags(m, flags)
