from json import loads
from typing import Tuple

from flask import Flask, jsonify, request

from v25.messaging.encoding import Encoding
from v25.messaging.message import Message
from v25.messaging.subject import Subject
from v25.storage.secstorage import SecureStorage
from v25.storage.storage import AbstractStorage


class API(Flask):
    def __init__(self, import_name, storage: AbstractStorage):
        self.storage = storage
        super().__init__(import_name)
        self.routes()

    def allowed(self, s: Subject):
        return 'allowed' in self.storage.check(s)

    def ss(self):
        d = request.json
        source: str = d['source']
        subject = Subject.loads(d['subject'])
        assert self.allowed(subject)
        subject.vkey.verify(source.encode(), Encoding.decode(d['signature']))
        return loads(source), SecureStorage(self.storage, subject)

    def routes(self):
        app = self

        @app.route('/')
        def root():
            return "V25API"

        @app.route('/check', methods=['POST'])
        def check():
            d, storage = self.ss()
            return jsonify(storage.check(Subject.loads(d)))

        @app.route('/push', methods=['POST'])
        def push():
            d, storage = self.ss()
            return jsonify(storage.push(Message.loads(d)))

        @app.route('/edit', methods=['POST'])
        def edit():
            d, storage = self.ss()
            return jsonify(storage.edit(Message.loads(d['old']), Message.loads(d['new'])))

        @app.route('/delete', methods=['POST'])
        def delete():
            d, storage = self.ss()
            return jsonify(storage.delete(Message.loads(d)))

        @app.route('/pull', methods=['POST'])
        def pull():
            d, storage = self.ss()
            pair: Tuple[Subject, Subject] = Subject.loads(d['pair'][0]), Subject.loads(d['pair'][1])
            return jsonify(list(map(Message.dumps, storage.pull(pair, d['params']))))

        @app.route('/flags', methods=['POST'])
        def flags():
            d, storage = self.ss()
            return jsonify(storage.flags(Message.loads(d['m']), d['flags']))
